import warnings
from typing import List

from decorator import decorator


def show_message(error_msg: str, warning_instead_of_error: bool) -> None:
    if warning_instead_of_error:
        warnings.warn(error_msg)
    else:
        raise TypeError(error_msg)


def manage_error_in_parameter_types(checks: List[int], names: tuple, args: tuple, hints: dict, warning_instead_of_error: bool) -> None:
    warning_text = ["Parameter's type mismatch"]
    for check, name, arg in zip(checks, names, args):
        if check == 1:
            warning_text.append("Parameter's name: {}, expected: {} but got: {}".format(name, hints[name], type(arg)))

    error_msg = "\n\t".join(warning_text)
    show_message(error_msg, warning_instead_of_error)


@decorator
def enforce_type_hints(f, disable: bool = False, warning_instead_of_error: bool = False, *args, **kw):
    if disable:
        return f

    hints = f.__annotations__
    names = f.__code__.co_varnames

    # Check the types of the parameters
    checks = [0 if isinstance(arg, hints[name]) else 1 for name, arg in zip(names, args)]

    if 1 in checks:
        manage_error_in_parameter_types(checks, names, args, hints, warning_instead_of_error)

    result = f(*args, **kw)

    # Check the type of the returned object
    if 'return' in hints and not isinstance(result, hints['return']):
        error_msg = "Return type mismatch, expected: {} but got: {}".format(hints['return'], type(result))
        show_message(error_msg, warning_instead_of_error)

    return result



from setuptools import setup
from os import path

# read the contents of your README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='type_asserter',  # How you named your package folder (MyLib)
    packages=['type_asserter'],  # Chose the same as "name"
    version='1.0.0',  # Start with a small number and increase it with every change you make
    license='MIT',  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description="This package enforces the functions' type hints at runtime, making sure that the types of the actual parameters and the return value match the types specified in the function signature.",  # Give a short description about your library
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Valerio Neri',  # Type in your name
    author_email='valerioneri1997@gmail.com',  # Type in your E-Mail
    url='https://github.com/ValerioNeriGit/type_asserter.git',  # Provide either the link to your github or to your website
    download_url='https://github.com/user/reponame/archive/type-asserter_v1.0.0.tar.gz',  # I explain this later on
    keywords=['TYPE', 'TYPES', 'ASSERTION', 'TYPE ASSERTION', 'TYPE CHECKER', ],  # Keywords that define your package best
    install_requires=[
        'decorator'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)
